@@ -36,6 +36,7 @@ impl Error {
             ErrorKind::ExpectedButGot(s) => format!("Expected {}", s).into(),
             ErrorKind::ExpectedSelectorText => "Expected a text for selector".into(),
             ErrorKind::UnterminatedBlockComment => "Unterminated block comment".into(),
+            ErrorKind::InvalidCharsetAtRule => "Invalid @charset at-rule".into(),
             ErrorKind::InvalidTypeSelector => "Invalid type selector".into(),
             ErrorKind::InvalidSelector => "Invalid selector".into(),
             ErrorKind::InvalidAttrName => "Invalid attribute name".into(),
@@ -67,6 +68,7 @@ pub enum ErrorKind {
     ExpectedButGot(&'static str),
     ExpectedSelectorText,
     UnterminatedBlockComment,
+    InvalidCharsetAtRule,
     InvalidTypeSelector,
     InvalidSelector,
     InvalidAttrName,
